/*
 *  ---------
 * |.##> <##.|  Open Smart Card Development Platform (www.openscdp.org)
 * |#       #|  
 * |#       #|  Copyright (c) 2015 CardContact Software & System Consulting
 * |'##> <##'|  Andreas Schwier, 32429 Minden, Germany (www.cardcontact.de)
 *  --------- 
 *  
 * Simple android demo application using the SmartCard-HSM
 *  
 */
package de.cardcontact.android.ocf.sample;

import java.io.File;
import java.io.PrintStream;
import java.math.BigInteger;
import java.security.Key;
import java.security.KeyFactory;
import java.security.MessageDigest;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.ECField;
import java.security.spec.ECFieldFp;
import java.security.spec.ECParameterSpec;
import java.security.spec.ECPoint;
import java.security.spec.EllipticCurve;
import java.security.spec.RSAPublicKeySpec;
import java.util.Enumeration;

import opencard.core.OpenCardException;
import opencard.core.service.CardRequest;
import opencard.core.service.CardServiceException;
import opencard.core.service.CardServiceFactory;
import opencard.core.service.CardServiceRegistry;
import opencard.core.service.SmartCard;
import opencard.core.terminal.CardTerminal;
import opencard.core.terminal.CardTerminalException;
import opencard.core.terminal.CardTerminalFactory;
import opencard.core.terminal.CardTerminalRegistry;
import opencard.core.util.HexString;
import opencard.opt.iso.fs.CardFilePath;
import opencard.opt.security.CHVCardService;
import de.cardcontact.opencard.factory.RemoteClientCardServiceFactory;
import de.cardcontact.opencard.factory.SmartCardHSMCardServiceFactory;
import de.cardcontact.opencard.service.remoteclient.RemoteClient;
import de.cardcontact.opencard.service.remoteclient.RemoteNotificationListener;
import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMCardService;
import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMECKey;
import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMECPrivateKeySpec;
import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMRSAKey;
import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMRSAPrivateKeySpec;
import de.cardcontact.opencard.utils.StreamingAPDUTracer;
import de.cardcontact.tlv.ConstructedTLV;
import de.cardcontact.tlv.ParseBuffer;
import de.cardcontact.tlv.PrimitiveTLV;
import de.cardcontact.tlv.TLVEncodingException;
import de.cardcontact.tlv.cvc.CardVerifiableCertificate;

import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Process;
import android.os.StrictMode;
import android.preference.PreferenceManager;
import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

public class MainActivity extends MainMenuActivity {
        private TextView view;
        private Thread cudThread = null;
        private SharedPreferences preferences;
        String[] tempPaths;

    @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            setContentView(R.layout.activity_main);

            ImageView img = (ImageView) findViewById(R.id.app_icon_hsm);
            img.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    Intent intent = new Intent();
                    intent.setAction(Intent.ACTION_VIEW);
                    intent.addCategory(Intent.CATEGORY_BROWSABLE);
                    intent.setData(Uri.parse("https://www.smartcard-hsm.com/"));
                    startActivity(intent);
                }
            });

            view = (TextView) findViewById(R.id.text);
            view.setMovementMethod(new ScrollingMovementMethod());
            view.setText("");

            preferences = PreferenceManager.getDefaultSharedPreferences(this);

            StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
            StrictMode.setThreadPolicy(policy);

            File[] tempDirs = this.getApplicationContext().getExternalCacheDirs();

            tempPaths = new String[tempDirs.length];
            for (int i = 0; i < tempDirs.length; i++) {
                tempPaths[i] = tempDirs[i].getAbsolutePath();
            }
        }

        //opens the OCF Crypto Testsuite Activity
        public void go_crypto_testsuite(View v) {
            TextView go_update_daemon = (TextView) findViewById(R.id.go_crypto_testsuite);
            Intent goToTestsuite = new Intent(this, TestsuiteActivity.class);
            startActivity(goToTestsuite);
        }

        //opens the OCF Crypto Testsuite Activity for the NFC reader
        public void go_crypto_testsuite_nfc(View v) {
            TextView go_update_daemon = (TextView) findViewById(R.id.go_crypto_testsuite_nfc);
            Intent goToNFCTestsuite = new Intent(this, TestsuiteNFCActivity.class);
            startActivity(goToNFCTestsuite);
        }

        //opens the OCF Crypto Testsuite Activity for the CCID reader
        public void go_crypto_testsuite_ccid(View v) {
            TextView go_update_daemon = (TextView) findViewById(R.id.go_crypto_testsuite_ccid);
            Intent goToCCIDTestsuite = new Intent(this, TestsuiteCCIDActivity.class);
            startActivity(goToCCIDTestsuite);
        }

        //opens the OCF Update Daemon Activity
        public void go_update_daemon(View v) {
            TextView go_update_daemon = (TextView) findViewById(R.id.go_update_daemon);
            Intent goToOCF = new Intent(this, OCFUpdateActivity.class);
            startActivity(goToOCF);
        }

        //opens the Settings Activity
        public void go_settings(View v) {
            TextView go_settings = (TextView) findViewById(R.id.go_settings);
            Intent goSettings = new Intent(this, SettingsActivity.class);
            startActivity(goSettings);
        }

        @Override
        public void onDestroy() {
            super.onDestroy();
        /*
         * Force the system to close the app down completely instead of
         * retaining it in the background. The virtual machine that runs the
         * app will be killed. The app will be completely created as a new
         * app in a new virtual machine running in a new process if the user
         * starts the app again.
         */
            System.exit(0);
        }

        @Override
        public void onStart() {
            super.onStart();
        }

        @Override
        public void onStop() {
            super.onStop();
        }

        public class Menu extends MainMenuActivity {

        }
}
