package de.cardcontact.android.ocf.sample;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Bundle;
import android.app.Activity;
import android.os.Process;
import android.preference.PreferenceManager;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import java.io.File;
import java.util.Enumeration;

import de.cardcontact.cli.CardUpdater;
import de.cardcontact.opencard.android.swissbit.SBMicroSDCardTerminalFactory;
import de.cardcontact.opencard.factory.RemoteClientCardServiceFactory;
import de.cardcontact.opencard.factory.SmartCardHSMCardServiceFactory;
import de.cardcontact.opencard.service.remoteclient.RemoteClient;
import de.cardcontact.opencard.service.remoteclient.RemoteNotificationListener;
import opencard.core.OpenCardException;
import opencard.core.service.CardRequest;
import opencard.core.service.CardServiceException;
import opencard.core.service.CardServiceFactory;
import opencard.core.service.CardServiceRegistry;
import opencard.core.service.SmartCard;
import opencard.core.terminal.CardTerminal;
import opencard.core.terminal.CardTerminalException;
import opencard.core.terminal.CardTerminalFactory;
import opencard.core.terminal.CardTerminalRegistry;
import opencard.opt.security.CHVCardService;

/**
 * Created by mbl on 07.02.2017.
 */

public class OCFUpdateActivity extends MainMenuActivity {
        private TextView view;
        private Thread cudThread = null;
        private SharedPreferences preferences;
        String[] tempPaths;


        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            setContentView(R.layout.activity_ocf_update);

            view = (TextView) findViewById(R.id.text);
            view.setMovementMethod(new ScrollingMovementMethod());
            view.setText("");

            preferences = PreferenceManager.getDefaultSharedPreferences(this);

            File[] tempDirs = this.getApplicationContext().getExternalCacheDirs();

            tempPaths = new String[tempDirs.length];
            for (int i = 0; i < tempDirs.length; i++) {
                tempPaths[i] = tempDirs[i].getAbsolutePath();
            }
        }

        //starting the OCF Update Daemon
        public void start(View v) {
            TextView start = (TextView) findViewById(R.id.start_update_daemon);
            view.setText("");
            CardUpdater cu = new CardUpdater();
            cu.execute((Integer) null);
        }

        //stopping the OCF Update Daemon
        public void stop(View v) {
            TextView stop = (TextView) findViewById(R.id.stop_update_deamon);
            //Used by activity_ocf_update
            TextView stop_update_daemon = (TextView) findViewById(R.id.stop_update_deamon);
            if (this.cudThread != null)
                this.cudThread.interrupt();

            try {
                view.append("Shutting down...\n");
                view.refreshDrawableState();
                SmartCard.shutdown();
            } catch (Exception e) {
                // Ignore
            }
        }

        public class CardUpdater extends AsyncTask<Object, Object, Object> implements RemoteNotificationListener {

            private String readerName = "sbCard";
            private String url = null;
            private String session = null;

            public CardUpdater() {
            }

            public void log(String s) {
                publishProgress(s);
            }

            @Override
            public void remoteNotify(int id, String message, int ttc) {
                publishProgress(message);
            }

            /**
             * Configure and start OCF
             *
             * @throws OpenCardException
             * @throws ClassNotFoundException
             */
            public void setupOCF() throws OpenCardException, ClassNotFoundException {
                Log.i("SmartCard-HSM", "OCF startup...");
                publishProgress("Starting OCF...");

                SmartCard.startup();
                publishProgress("OCF started");

                Log.i("SmartCard-HSM", "Creating card terminal registry...");
                CardTerminalRegistry ctr = CardTerminalRegistry.getRegistry();
                SBMicroSDCardTerminalFactory sbcardf = new SBMicroSDCardTerminalFactory(getApplicationContext());
            /*
             * Initialize the card terminal providing paths to external storage locations
             */
                sbcardf.createCardTerminals(ctr, null);
                publishProgress("Created card terminal registry");

                Log.i("SmartCard-HSM", "Creating card service registry...");
                CardServiceRegistry csr = CardServiceRegistry.getRegistry();
                publishProgress("Created card service registry");

                Log.i("CardUpdaterDaemon", "Adding SmartCard-HSM card service...");
                CardServiceFactory csf = new SmartCardHSMCardServiceFactory();
                csr.add(csf);
                publishProgress("Added SmartCard-HSM card service");

                Log.i("CardUpdaterDaemon", "Adding Remote Client card service...");
                csf = new RemoteClientCardServiceFactory();
                csr.add(csf);
                publishProgress("Added Remote Client card service");

            }

            /**
             * Perform update
             *
             * @param args command line arguments
             */
            public void run(String[] args) {

                try {
                    setupOCF();
                    publishProgress(SmartCard.getVersion());

                    CardTerminalRegistry ctr = CardTerminalRegistry.getRegistry();
                    Enumeration ctlist = ctr.getCardTerminals();

                    publishProgress("Available card terminals:");
                    while (ctlist.hasMoreElements()) {
                        CardTerminal ct = (CardTerminal) ctlist.nextElement();
                        publishProgress(" " + ct.getName());
                    }
                    publishProgress("");

                    CardTerminal ct = null;

                    ct = ctr.cardTerminalForName(readerName);

                    if (ct == null) {
                        publishProgress("Card reader " + readerName + " not found");
                        return;
                    }

                    String pin = preferences.getString("pin", "648219");

                    if (!verifyPin(ct, 0, pin.getBytes())) {
                        publishProgress("PIN verification failed...");
                        return;
                    }

                    if (url == null) {
                        CardUpdaterDaemon cud = new CardUpdaterDaemon(this, ct);
                        cudThread = new Thread(cud);
                        cudThread.setDaemon(true);
                        cudThread.setName("Card Update Daemon Thread");
                        cudThread.start();
                    } else {
                        publishProgress(1, "Connecting to " + url);

                        CardRequest cr = new CardRequest(CardRequest.ANYCARD, ct, RemoteClient.class);
                        cr.setTimeout(0);
                        SmartCard sc = SmartCard.waitForCard(cr);
                        if (sc == null) {
                            publishProgress(1, "No card in reader");
                        } else {
                            RemoteClient rc = (RemoteClient) sc.getCardService(RemoteClient.class, true);

                            rc.update(url, session, this);

                            publishProgress(1, "Update complete");

                            sc.close();
                        }

                    }

                } catch (Exception e) {
                    publishProgress("Error: " + e.getLocalizedMessage() + "\n");
                    e.printStackTrace();
                    Log.i("SmartCard-HSM", e.getLocalizedMessage());
                }
            }

            /**
             * Perform PIN verification
             *
             * @param chvNumber the CHV number used for PIN verification
             * @return true if verification was successful
             * @throws CardTerminalException
             * @throws CardServiceException
             * @throws ClassNotFoundException
             */
            boolean verifyPin(CardTerminal ct, int chvNumber, byte[] password) throws CardTerminalException, CardServiceException, ClassNotFoundException {
                CardRequest creq = new CardRequest(CardRequest.ANYCARD, ct, CHVCardService.class);
                creq.setTimeout(0);

                SmartCard card = SmartCard.waitForCard(creq);
                if (card == null) {
                    publishProgress("No card in reader");
                    return false;
                }
                CHVCardService chv = (CHVCardService) card.getCardService(CHVCardService.class, true);

                boolean verified = false;
                verified = chv.verifyPassword(null, chvNumber, password);

                publishProgress("PIN verified: " + verified);
                return verified;
            }

            @Override
            protected Object doInBackground(Object... params) {
                run(null);
                return null;
            }

            @Override
            protected void onProgressUpdate(Object... values) {
                for (Object s : values) {
                    view.append(s + "\n");
                }

                view.refreshDrawableState();
            }
        }
    }

