package de.cardcontact.android.ocf.sample;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.nfc.NfcAdapter;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.io.PrintStream;
import java.math.BigInteger;
import java.security.Key;
import java.security.KeyFactory;
import java.security.MessageDigest;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.ECField;
import java.security.spec.ECFieldFp;
import java.security.spec.ECParameterSpec;
import java.security.spec.ECPoint;
import java.security.spec.EllipticCurve;
import java.security.spec.RSAPublicKeySpec;

import de.cardcontact.opencard.android.nfc.NfcCardTerminal;
import de.cardcontact.opencard.android.nfc.NfcCardTerminalFactory;
import de.cardcontact.opencard.factory.SmartCardHSMCardServiceFactory;
import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMCardService;
import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMECKey;
import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMECPrivateKeySpec;
import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMRSAKey;
import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMRSAPrivateKeySpec;
import de.cardcontact.opencard.utils.StreamingAPDUTracer;
import de.cardcontact.tlv.ConstructedTLV;
import de.cardcontact.tlv.ParseBuffer;
import de.cardcontact.tlv.PrimitiveTLV;
import de.cardcontact.tlv.TLVEncodingException;
import de.cardcontact.tlv.cvc.CardVerifiableCertificate;
import opencard.core.service.CardRequest;
import opencard.core.service.CardServiceException;
import opencard.core.service.CardServiceFactory;
import opencard.core.service.CardServiceRegistry;
import opencard.core.service.SmartCard;
import opencard.core.terminal.CardTerminalFactory;
import opencard.core.terminal.CardTerminalRegistry;
import opencard.core.util.HexString;
import opencard.opt.iso.fs.CardFilePath;

/**
 * Created by mbl on 07.02.2017.
 */

public class TestsuiteNFCActivity extends MainMenuActivity {
        private TestsuiteNFCActivity MainMenuActivity;
        private TextView view;
        private Thread cudThread = null;
        private SharedPreferences preferences;
        String[] tempPaths;
        AndroidCardService acs;
        boolean mBound = false;

        /** Defines callbacks for service binding, passed to bindService() */
        private ServiceConnection connection = new ServiceConnection() {

            @Override
            public void onServiceConnected(ComponentName className,
                                           IBinder service) {
                // We've bound to LocalService, cast the IBinder and get LocalService instance
                AndroidCardService.LocalBinder binder = (AndroidCardService.LocalBinder) service;
                acs = binder.getService();
                mBound = true;
            }

            @Override
            public void onServiceDisconnected(ComponentName arg0) {
                mBound = false;
            }
        };

        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            setContentView(R.layout.activity_testsuite);

            view = (TextView) findViewById(R.id.text);
            view.setMovementMethod(new ScrollingMovementMethod());
            view.setText("");

            File[] tempDirs = this.getApplicationContext().getExternalCacheDirs();

            preferences = PreferenceManager.getDefaultSharedPreferences(this);

            tempPaths = new String[tempDirs.length];
            for (int i = 0; i < tempDirs.length; i++) {
                tempPaths[i] = tempDirs[i].getAbsolutePath();
            }

            // use this to start and trigger a service
            Intent i= new Intent(this.getApplicationContext(), AndroidCardService.class);
            // potentially add data to the intent
            i.putExtra("KEY1", "Value to be used by the service");
            this.getApplicationContext().bindService(i, connection, Context.BIND_AUTO_CREATE);

        }

        private void enableReaderMode() {
            NfcAdapter adapter = NfcAdapter.getDefaultAdapter(this);

            Bundle bundle = new Bundle();
            //bundle.putInt(NfcAdapter.EXTRA_READER_PRESENCE_CHECK_DELAY, timeout * 10);
            adapter.enableReaderMode(this, acs,
                    NfcAdapter.FLAG_READER_NFC_A | NfcAdapter.FLAG_READER_SKIP_NDEF_CHECK,
                    bundle);
        }


        private void disableReaderMode() {
            NfcAdapter nfc = NfcAdapter.getDefaultAdapter(this);
            if (nfc != null) {
                nfc.disableReaderMode(this);
            }
        }

         //starting the OCF Testsuite
        public void run_ocf_testsuite(View v) {
            TextView run_ocf_testsuite = (TextView) findViewById(R.id.run_crypto_testsuite);
            try {
                CryptoTestDaemon cad = new CryptoTestDaemon();
                cad.execute((Integer) null);

            } catch (Exception e) {
                view.append("Error: " + e.getLocalizedMessage() + "\n");
                Log.i("SmartCard-HSM", e.getLocalizedMessage());
            }
        }


    private class CryptoTestDaemon extends AsyncTask<Object, Object, Object> {

            @Override
            protected Object doInBackground(Object... params) {

                try {

                    enableReaderMode();

                    Log.i("OCF Sample", "OCF startup...");
                    publishProgress("Starting OCF...");

                    SmartCard.startup();
                    publishProgress("OCF started");

                    Log.i("SmartCard-HSM", "Creating card terminal registry...");
                    CardTerminalRegistry ctr = CardTerminalRegistry.getRegistry();
                    CardTerminalFactory ctf = new NfcCardTerminalFactory(NfcAdapter.getDefaultAdapter(getApplicationContext()));
                    ctf.createCardTerminals(ctr, null);
                    publishProgress("Created card terminal registry");

                    Log.i("SmartCard-HSM", "Creating card service registry...");
                    CardServiceRegistry csr = CardServiceRegistry.getRegistry();
                    publishProgress("Created card service registry");

                    Log.i("CardUpdaterDaemon", "Adding SmartCard-HSM card service...");
                    CardServiceFactory csf = new SmartCardHSMCardServiceFactory();
                    csr.add(csf);
                    publishProgress("Added SmartCard-HSM card service");

                    Log.i("SmartCard-HSM", "Creating card request...");
                    CardRequest cr = new CardRequest(CardRequest.ANYCARD, null, SmartCardHSMCardService.class);
                    publishProgress("Waiting for card");
                    SmartCard sc = SmartCard.waitForCard(cr);
                    if (sc == null) {
                        Log.i("SmartCard-HSM", "Could not get smart card...");
                        publishProgress("Could not get card - aborting");
                        return null;
                    }

                    sc.setAPDUTracer(new StreamingAPDUTracer(new PrintStream(new LogCatOutputStream())));

                    publishProgress("Card found");

                    Log.i("SmartCard-HSM", "Trying to create card service...");
                    SmartCardHSMCardService schsmcs = (SmartCardHSMCardService) sc.getCardService(SmartCardHSMCardService.class, true);
                    Log.i("SmartCard-HSM", "Got SmartCard HSM card service!");
                    publishProgress("Got SmartCard HSM card service!");

                    Log.i("SmartCard-HSM", "Card ATR:" + HexString.hexify(schsmcs.getCard().getCardID().getATR()));
                    publishProgress("ATR:" + HexString.hexify(schsmcs.getCard().getCardID().getATR()) + "\n");

                    byte[] filelist = schsmcs.enumerateObjects();
                    publishProgress("Files: " + HexString.hexify(filelist) + "\n");

                    // Read and dump the device certificate
                    publishProgress("Reading device certificate...");
                    CardFilePath ef_2f02 = new CardFilePath(":2F02");
                    byte[] devcert = schsmcs.read(ef_2f02, 0, 0);
                    publishProgress("Device certificate (" + devcert.length + " bytes): \n" + HexString.hexify(devcert) + "\n\n");

                    // Verify the PIN
                    publishProgress("Verifying PIN...");

                    String pin = preferences.getString("pin", "648219");
                    schsmcs.verifyPassword(null, 0, pin.getBytes());

                    // Copy the device certificate to another file
                    publishProgress("Writing device certificate to EF EE0F...");
                    CardFilePath ef_ee0f = new CardFilePath(":EE0F");
                    schsmcs.write(ef_ee0f, 0, devcert);

                    // Delete existing key pair - if present
                    publishProgress("Deleting possibly existing key with id 0x01...");
                    CardFilePath keyfile_01 = new CardFilePath(":CC01");
                    try {
                        schsmcs.delete(keyfile_01);
                    } catch (CardServiceException cse) {
                        // ignore - file not present
                    }

                    // Create RSA key
                    publishProgress("Generating 2048 bit RSA key with key id 0x01...");
                    SmartCardHSMRSAPrivateKeySpec rsaspec = new SmartCardHSMRSAPrivateKeySpec("detest0000", "detest0001", 65537, 2048);
                    rsaspec.setAlgorithm(new byte[]{0x04, 0x00, 0x7F, 0x00, 0x07, 0x02, 0x02, 0x02, 0x01, 0x02}); // PKCS1-v1-5-SHA-256
                    byte[] d = schsmcs.generateKeyPair((byte) 0x01, (byte) 0x00, rsaspec);
                    publishProgress("RSA CSR (" + d.length + " bytes): \n" + HexString.hexify(d) + "\n\n");

                    RSAPublicKeySpec rsaPublicKeySpec = getRSAPublicKeySpec(d);

                    // Sign with RSA v1.5 - we are performing the hashing outside the card
                    byte[] data = "HEEEEEEEEEEEEEEEEEEEEEEEELLLLLLLLLLLLLLLLLLLLLOOOOOOOOOO".getBytes();
                    MessageDigest sha256 = MessageDigest.getInstance("SHA-256");
                    sha256.update(data);
                    byte[] digest = sha256.digest();
                    SmartCardHSMRSAKey rsa2048Key = new SmartCardHSMRSAKey((byte) 0x01, "RSA-v1-5-SHA-256", (short) 2048);
                    byte[] sig = schsmcs.signHash(rsa2048Key, "SHA256withRSA", "PKCS1_V15", digest);

                    // Verify RSA Signature
                    Key verifyKey = KeyFactory.getInstance("RSA").generatePublic(rsaPublicKeySpec);
                    Signature verifier = Signature.getInstance("SHA256withRSA");
                    verifier.initVerify((PublicKey) verifyKey);
                    verifier.update(data);
                    boolean sigValid = verifier.verify(sig);
                    publishProgress("RSA signature is " + (sigValid ? "valid" : "invalid") + "\n\n");

                    // Delete existing key pair - if present
                    publishProgress("Deleting possibly existing key with id 0x02...");
                    CardFilePath keyfile_02 = new CardFilePath(":CC02");
                    try {
                        schsmcs.delete(keyfile_02);
                    } catch (CardServiceException cse) {
                        // ignore - file not present
                    }

                    // Create ECC key
                    BigInteger prime = new BigInteger("A9FB57DBA1EEA9BC3E660A909D838D726E3BF623D52620282013481D1F6E5377", 16);
                    ECField field = new ECFieldFp(prime);
                    BigInteger a, b;
                    a = new BigInteger("7D5A0975FC2C3057EEF67530417AFFE7FB8055C126DC5C6CE94A4B44F330B5D9", 16);
                    b = new BigInteger("26DC5C6CE94A4B44F330B5D9BBD77CBF958416295CF7E1CE6BCCDC18FF8C07B6", 16);
                    EllipticCurve curve = new EllipticCurve(field, a, b);

                    BigInteger x = new BigInteger("8BD2AEB9CB7E57CB2C4B482FFC81B7AFB9DE27E1E3BD23C23A4453BD9ACE3262", 16);
                    BigInteger y = new BigInteger("547EF835C3DAC4FD97F8461A14611DC9C27745132DED8E545C1D54C72F046997", 16);
                    ECPoint g = new ECPoint(x, y);
                    BigInteger n = new BigInteger("A9FB57DBA1EEA9BC3E660A909D838D718C397AA3B561A6F7901E0E82974856A7", 16);
                    int h = 1;
                    ECParameterSpec ecparam = new ECParameterSpec(curve, g, n, h);

                    publishProgress("Generating 256 bit ECC key with key id 0x02...");
                    SmartCardHSMECPrivateKeySpec eccspec = new SmartCardHSMECPrivateKeySpec("detest0000", "detest0002", ecparam);
                    eccspec.setAlgorithm(new byte[]{0x04, 0x00, 0x7F, 0x00, 0x07, 0x02, 0x02, 0x02, 0x02, 0x03}); // ECDSA-SHA-256
                    d = schsmcs.generateKeyPair((byte) 0x02, (byte) 0x00, eccspec);
                    publishProgress("ECC CSR (" + d.length + " bytes): \n" + HexString.hexify(d) + "\n\n");

                    // Sign with ECDSA - we are performing the hashing outside the card
                    publishProgress("Creating ECC signature...");
                    CardVerifiableCertificate cvc = new CardVerifiableCertificate("CVC", d);
                    data = "HEEEEEEEEEEEEEEEEEEEEEEEELLLLLLLLLLLLLLLLLLLLLOOOOOOOOOO".getBytes();
                    sha256 = MessageDigest.getInstance("SHA-256");
                    sha256.update(data);
                    digest = sha256.digest();
                    byte[] digestPad = new byte[32];
                    System.arraycopy(digest, 0, digestPad, 32 - digest.length, digest.length);
                    SmartCardHSMECKey ecc256Key = new SmartCardHSMECKey((byte) 0x02, "ECDSA-SHA-256", (short) 256);
                    sig = schsmcs.signHash(ecc256Key, "NONEwithECDSA", "NONE", digest);
                    publishProgress("ECC Signature (" + sig.length + " bytes): \n" + HexString.hexify(sig) + "\n\n");

                    //Verify Signature
                    publishProgress("Verifying ECC signature...");
                    verifyKey = cvc.getPublicKey();
                    Signature signature = Signature.getInstance("SHA256withECDSA");
                    signature.initVerify((PublicKey) verifyKey);
                    signature.update(data);
                    sigValid = signature.verify(sig);
                    publishProgress("ECC signature is " + (sigValid ? "valid" : "invalid"));

                    disableReaderMode();

                } catch (Exception e) {
                    publishProgress("Error: " + e.getLocalizedMessage() + "\n");
                    e.printStackTrace();
                    Log.i("SmartCard-HSM", e.getLocalizedMessage());
                } finally {
                    try {
                        SmartCard.shutdown();
                        disableReaderMode();
                    } catch (Exception e) {
                        Log.i("SmartCard-HSM", e.getLocalizedMessage());
                    }
                }

                return null;
            }

            private ConstructedTLV getPublicKeyFromCertificate(byte[] cert) throws TLVEncodingException {
                ParseBuffer pb = new ParseBuffer(cert);
                ConstructedTLV tlv = new ConstructedTLV(pb);
                ConstructedTLV cvc = (ConstructedTLV) tlv.get(0);
                ConstructedTLV body = (ConstructedTLV) cvc.get(0);
                ConstructedTLV pk = (ConstructedTLV) body.get(2);

                return pk;
            }

            private BigInteger byteArrayToUnsignedBigInteger(byte[] data) {
                byte[] absoluteValue = new byte[data.length + 1];
                System.arraycopy(data, 0, absoluteValue, 1, data.length);
                return new BigInteger(absoluteValue);
            }

            private RSAPublicKeySpec getRSAPublicKeySpec(byte[] data) throws TLVEncodingException {
                ConstructedTLV pk = getPublicKeyFromCertificate(data);

                byte[] mod = pk.get(1).getValue();
                BigInteger modulus = byteArrayToUnsignedBigInteger(mod);

                PrimitiveTLV exp = (PrimitiveTLV) pk.get(2);
                BigInteger exponent = byteArrayToUnsignedBigInteger(exp.getValue());

                return new RSAPublicKeySpec(modulus, exponent);
            }

            @Override
            protected void onPreExecute() {
                view.setText("Starting demo...");
                super.onPreExecute();
            }

            @Override
            protected void onPostExecute(Object result) {
                Toast.makeText(getApplicationContext(), "Crypto Testsuite finished", Toast.LENGTH_SHORT).show();
                super.onPostExecute(result);
            }

            @Override
            protected void onProgressUpdate(Object... values) {
                for (Object s : values) {
                    view.append(s + "\n");
                }

                view.refreshDrawableState();
            }
        }
    }

