/*
 * Copyright (c) 2020 CardContact Systems GmbH, Minden, Germany.
 *
 * Redistribution and use in source (source code) and binary (object code)
 * forms, with or without modification, are permitted provided that the
 * following conditions are met:
 * 1. Redistributed source code must retain the above copyright notice, this
 * list of conditions and the disclaimer below.
 * 2. Redistributed object code must reproduce the above copyright notice,
 * this list of conditions and the disclaimer below in the documentation
 * and/or other materials provided with the distribution.
 * 3. The name of CardContact may not be used to endorse or promote products derived
 * from this software or in any other form without specific prior written
 * permission from CardContact.
 * 4. Redistribution of any modified code must be labeled "Code derived from
 * the original OpenCard Framework".
 *
 * THIS SOFTWARE IS PROVIDED BY CardContact "AS IS" FREE OF CHARGE. CardContact SHALL NOT BE
 * LIABLE FOR INFRINGEMENTS OF THIRD PARTIES RIGHTS BASED ON THIS SOFTWARE.  ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  CardContact DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THIS
 * SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR THAT THE OPERATION OF IT WILL
 * BE UNINTERRUPTED OR ERROR-FREE.  IN NO EVENT, UNLESS REQUIRED BY APPLICABLE
 * LAW, SHALL CardContact BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  ALSO, CardContact IS UNDER NO OBLIGATION
 * TO MAINTAIN, CORRECT, UPDATE, CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS
 * SOFTWARE.
 */

package de.cardcontact.opencard.android.ccid;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

public class CcidAPDU extends CcidUSB {


    public CcidAPDU(USBDeviceHandler usbHandler) {
        super(usbHandler);
    }


    public void ccidAPDUInit () throws IOException {
    }


    public byte[] ccidAPDUProcess (byte[] cmd) throws IOException {
        int rc, r, maxlr, lr, lc;
        int len, po = 0;
        byte[] buf = new byte[BUFFMAX];
        byte[] rsp = null;
        byte[] status = new byte[1];
        byte[] error = new byte[1];
        byte[] chain = new byte[1];
        short level = 0;
        ByteArrayOutputStream bos = new ByteArrayOutputStream();

    	lr = 0;
    	lc = cmd.length;
        while (lc > 0) {
            len = lc;
            if (lc > BUFFMAX) {
                if (level > 0)
                    level = 3;			// Intermediate extended command
                else
                    level = 1;			// First extended command
                len = BUFFMAX;
            } else {
                if (level > 0)
                    level = 2;			// Final extended command
            }

            PC_to_RDR_XfrBlock(cmd, po, len, level);

            lc -= len;
            po += len;

            len = BUFFMAX;
            rsp = RDR_to_PC_DataBlock(status, error, chain);
        }

        r = 0;
        while (true) {
            /*
            if (len > maxlr) {
                len = maxlr;
                r = ERR_MEMORY;
            }
            */

            bos.write(rsp);

            if ((chain[0] == 1) || (chain[0] == 3)) {
                PC_to_RDR_XfrBlock(null, 0, 0, 0x10);

                rsp = RDR_to_PC_DataBlock(status, error, chain);
                continue;
            }
            break;
        }

        return bos.toByteArray();
      }
}
