/*
 * Copyright (c) 2020 CardContact Systems GmbH, Minden, Germany.
 *
 * Redistribution and use in source (source code) and binary (object code)
 * forms, with or without modification, are permitted provided that the
 * following conditions are met:
 * 1. Redistributed source code must retain the above copyright notice, this
 * list of conditions and the disclaimer below.
 * 2. Redistributed object code must reproduce the above copyright notice,
 * this list of conditions and the disclaimer below in the documentation
 * and/or other materials provided with the distribution.
 * 3. The name of CardContact may not be used to endorse or promote products derived
 * from this software or in any other form without specific prior written
 * permission from CardContact.
 * 4. Redistribution of any modified code must be labeled "Code derived from
 * the original OpenCard Framework".
 *
 * THIS SOFTWARE IS PROVIDED BY CardContact "AS IS" FREE OF CHARGE. CardContact SHALL NOT BE
 * LIABLE FOR INFRINGEMENTS OF THIRD PARTIES RIGHTS BASED ON THIS SOFTWARE.  ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  CardContact DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THIS
 * SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR THAT THE OPERATION OF IT WILL
 * BE UNINTERRUPTED OR ERROR-FREE.  IN NO EVENT, UNLESS REQUIRED BY APPLICABLE
 * LAW, SHALL CardContact BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  ALSO, CardContact IS UNDER NO OBLIGATION
 * TO MAINTAIN, CORRECT, UPDATE, CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS
 * SOFTWARE.
 */

package de.cardcontact.opencard.android.ccid;

import android.hardware.usb.UsbConstants;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbEndpoint;
import android.hardware.usb.UsbInterface;
import android.hardware.usb.UsbManager;

import java.io.IOException;
import java.util.List;

public class USBDeviceHandler {

    protected UsbDeviceConnection connection;
    protected UsbInterface usbinterface;
    protected UsbEndpoint pipeIn;
    protected UsbEndpoint pipeOut;
    protected int maxIn;
    protected int maxOut;
    protected boolean apduMode = true;

    /**
     * Constructor for DeviceHandler</br>
     * For now we assume there's only this single card
     * On API there might be another parameter for the desired slot
     */
    public USBDeviceHandler(UsbDevice usbdevice, UsbManager usbmanager) {
        connection = usbmanager.openDevice(usbdevice);

        if (connection == null) {
            throw new IllegalArgumentException("Could not connect to USB CCID device");
        }

        //Lets assume there's only a single interface on the knows smartcards for now
        UsbInterface usbI = usbdevice.getInterface(0);
        usbinterface = usbI;
        int end = usbI.getEndpointCount();
        for (int i=0; i < end; i++) {
            UsbEndpoint usbE = usbI.getEndpoint(i);
            int addr = usbE.getAddress();
            int dir = usbE.getDirection();
            int type = usbE.getType();

            if (type == UsbConstants.USB_ENDPOINT_XFER_BULK) {
                if (dir == 0x00) {
                    pipeOut = usbE;
                } else if(dir == 0x80) {
                    pipeIn = usbE;
                }
            }
        }

        maxIn = pipeIn.getMaxPacketSize();
        maxOut = pipeOut.getMaxPacketSize();

        byte[] descriptors = connection.getRawDescriptors();

        try {
            CcidDescriptor d = CcidDescriptor.parse(descriptors);

            if (d.getFeatures().contains(CcidDescriptor.Feature.TPDU)) {
                    apduMode = false;
            }
        } catch (IOException ioe) {
                throw new IllegalArgumentException("Illegal CCID device descriptor");
        }
    }


    public static boolean isSupported(UsbDevice usbdevice) {
        return Device.exists(usbdevice.getVendorId(), usbdevice.getProductId());
    }
}
