/*
 *  ---------
 * |.##> <##.|  Open Smart Card Development Platform (www.openscdp.org)
 * |#       #|  
 * |#       #|  Copyright (c) 1999-2016 CardContact Systems GmbH
 * |'##> <##'|  32429 Minden, Germany (www.cardcontact.de)
 *  --------- 
 *
 *  This file is part of OpenSCDP.
 *
 *  OpenSCDP is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 *
 *  OpenSCDP is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with OpenSCDP; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.cardcontact.jceprovdemo;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SecureRandom;
import java.security.Security;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.LoginException;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import de.cardcontact.smartcardhsmprovider.SmartCardHSMProvider;

/*
 * Simple demo showing the usage of the SmartCard-HSM provider for
 * SSL/TLS client authentication.
 * 
 */

public class SSLDemo {

	
	private static void initProvider() {
		MyPasswordCallbackHandler pch = new MyPasswordCallbackHandler();	  		
		SmartCardHSMProvider provider = new SmartCardHSMProvider();

		// make sure to add the BC provider first
		Security.addProvider(new BouncyCastleProvider());
		
		try {
			provider.login(null, (CallbackHandler) pch);
		} catch (LoginException e) {
			e.printStackTrace();
			System.exit(-1);
		}
		
		Security.insertProviderAt(provider, 0);

	}

	
	public static void main(String[] args) throws KeyStoreException, NoSuchProviderException, NoSuchAlgorithmException, CertificateException, IOException, UnrecoverableKeyException, KeyManagementException {
		initProvider();
		
		// Initialize the key and trust material.
		KeyStore ksKeys = KeyStore.getInstance("SmartCardHSMKeyStore", "SmartCardHSM");
		ksKeys.load(null, null);
		
// Enable you own truststore is the server SSL certificate is not from a public SSL CA
//		KeyStore ksTrust = KeyStore.getInstance("JKS");
//		InputStream in = new FileInputStream("lib/truststore.jks");
//		ksTrust.load(in, "openscdp".toCharArray());
		
		// KeyManager's decide which key material to use.
		KeyManagerFactory kmf = 
			KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());

		kmf.init(ksKeys, null);
		
		// TrustManager's decide whether to allow connections.
//		TrustManagerFactory tmf =
//		    TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
//		tmf.init(ksTrust);
		
		SecureRandom rng = SecureRandom.getInstance("NativePRNG", "SmartCardHSM");
		
		SSLContext sslContext = SSLContext.getInstance("TLSv1");
//		sslContext.init(kmf.getKeyManagers(), tmf.getTrustManagers(), rng); 
		sslContext.init(kmf.getKeyManagers(), null, rng); 
				
		URL url = new URL("https://www.openehic.org/cgi-bin/ssllogin.pl");
		URLConnection c = url.openConnection();
		HttpsURLConnection sslc = (HttpsURLConnection)c;
		sslc.setSSLSocketFactory(sslContext.getSocketFactory());

		// read the data from the socket
		InputStream is = sslc.getInputStream();
		StringBuffer str = new StringBuffer();

		BufferedReader reader = new BufferedReader(new InputStreamReader(is));
		String line;
		
		while ((line = reader.readLine()) != null) {
			str.append(line);
			str.append('\n');
		}
		
		is.close();
		System.out.println(str.toString());

	}
}
