/*
 *  ---------
 * |.##> <##.|  Open Smart Card Development Platform (www.openscdp.org)
 * |#       #|
 * |#       #|  Copyright (c) 1999-2016 CardContact Systems GmbH
 * |'##> <##'|  32429 Minden, Germany (www.cardcontact.de)
 *  ---------
 *
 *  This file is part of OpenSCDP.
 *
 *  OpenSCDP is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 *
 *  OpenSCDP is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with OpenSCDP; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.cardcontact.smartcardhsmprovider;

import java.security.ProviderException;
import java.security.SecureRandomSpi;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMCardService;
import opencard.core.service.CardServiceException;
import opencard.core.terminal.CardTerminalException;

/**
 * <p>
 * This class provides a random number generator.
 * </p>
 * <p>
 * Initial seeding is not required.
 * </p>
 *
 * @author lew
 */
public class SecureRandom extends SecureRandomSpi {

    private final static Logger log = LoggerFactory.getLogger(SecureRandom.class);

    private static final long serialVersionUID = -1130070973762733185L;

    private SmartCardHSMProvider provider;

    /**
     * This object provides access to the SmartCard
     */
    private SmartCardHSMCardService schsm;

    /**
     * @param provider the SmartCardHSMProvider
     * @param algorithm this parameter is not in use
     */
    public SecureRandom(SmartCardHSMProvider provider, String algorithm) {
    	this.provider = provider;
    	this.schsm = provider.getSmartCardHSMCardService();
    }

    /**
     * @return the seed bytes or null
     */
    @Override
    protected byte[] engineGenerateSeed(int numBytes) {
	byte[] seed = null;

	try {
	    seed = schsm.generateRandom(numBytes);
	} catch (CardTerminalException e) {
	    log.error(e.getLocalizedMessage(), e);
	    throw new ProviderException(e);
	} catch (CardServiceException e) {
	    log.error(e.getLocalizedMessage(), e);
	    throw new ProviderException(e);
	}

	return seed;
    }

    @Override
    protected void engineNextBytes(byte[] bytes) {
	try {
	    byte[] rnd = schsm.generateRandom(bytes.length);
	    System.arraycopy(rnd, 0, bytes, 0, rnd.length);
	} catch (CardTerminalException e) {
	    log.error(e.getLocalizedMessage(), e);
	    throw new ProviderException(e);
	} catch (CardServiceException e) {
	    log.error(e.getLocalizedMessage(), e);
	    throw new ProviderException(e);
	}
    }

    /**
     * Seeding is not required.
     *
     * @deprecated
     */
    @Override
    protected void engineSetSeed(byte[] seed) {
    }

}
