/*
 *  ---------
 * |.##> <##.|  Open Smart Card Development Platform (www.openscdp.org)
 * |#       #|
 * |#       #|  Copyright (c) 1999-2016 CardContact Systems GmbH
 * |'##> <##'|  32429 Minden, Germany (www.cardcontact.de)
 *  ---------
 *
 *  This file is part of OpenSCDP.
 *
 *  OpenSCDP is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 *
 *  OpenSCDP is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with OpenSCDP; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.cardcontact.smartcardhsmprovider;

import java.security.spec.AlgorithmParameterSpec;

import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMKey;
import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMKeySpec;
import de.cardcontact.opencard.service.smartcardhsm.SmartCardHSMPrivateKeySpec;

/**
 * This class contains the label, keyspec and the signing key reference needed for key generation
 *
 * @author lew
 *
 */
public class SmartCardHSMParameterSpec implements AlgorithmParameterSpec {

	/**
	 * The domain parameter
	 */
	private SmartCardHSMKeySpec keySpec;

	/**
	 * The alias of the key
	 */
	private String label;

	/**
	 * JCE key paramater specification
	 *
	 * @param label the alias
	 * @param keySpec the domain parameter
	 */
	public SmartCardHSMParameterSpec(String label, SmartCardHSMKeySpec keySpec) {
		this.label = label;
		this.keySpec = keySpec;
	}

	/**
	 * JCE key paramater specification
	 *
	 * @param label the alias
	 * @param keySpec the domain parameter
	 * @deprecated Signing with key other than PrK.DevAur dropped in firmware 3.0
	 */
	public SmartCardHSMParameterSpec(String label, SmartCardHSMKeySpec keySpec, SmartCardHSMKey signingKey) {
		this.label = label;
		this.keySpec = keySpec;
	}

	public SmartCardHSMKeySpec getKeySpec() {
		return keySpec;
	}

	public void setKeySpec(SmartCardHSMPrivateKeySpec keySpec) {
		this.keySpec = keySpec;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}
}
